// Copyright 2003 Nokia Corporation.
//
// THIS SOURCE CODE IS PROVIDED 'AS IS', WITH NO WARRANTIES WHATSOEVER,
// EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY, FITNESS
// FOR ANY PARTICULAR PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE
// OR TRADE PRACTICE, RELATING TO THE SOURCE CODE OR ANY WARRANTY OTHERWISE
// ARISING OUT OF ANY PROPOSAL, SPECIFICATION, OR SAMPLE AND WITH NO
// OBLIGATION OF NOKIA TO PROVIDE THE LICENSEE WITH ANY MAINTENANCE OR
// SUPPORT. FURTHERMORE, NOKIA MAKES NO WARRANTY THAT EXERCISE OF THE
// RIGHTS GRANTED HEREUNDER DOES NOT INFRINGE OR MAY NOT CAUSE INFRINGEMENT
// OF ANY PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OWNED OR CONTROLLED
// BY THIRD PARTIES
//
// Furthermore, information provided in this source code is preliminary,
// and may be changed substantially prior to final release. Nokia Corporation
// retains the right to make changes to this source code at
// any time, without notice. This source code is provided for informational
// purposes only.
//
// Nokia and Nokia Connecting People are registered trademarks of Nokia
// Corporation.
// Java and all Java-based marks are trademarks or registered trademarks of
// Sun Microsystems, Inc.
// Other product and company names mentioned herein may be trademarks or
// trade names of their respective owners.
//
// A non-exclusive, non-transferable, worldwide, limited license is hereby
// granted to the Licensee to download, print, reproduce and modify the
// source code. The licensee has the right to market, sell, distribute and
// make available the source code in original or modified form only when
// incorporated into the programs developed by the Licensee. No other
// license, express or implied, by estoppel or otherwise, to any other
import java.util.*;
import javax.microedition.lcdui.*;
import javax.microedition.m3g.*;
import javax.microedition.midlet.*;

public class Maze3DMIDlet
  extends MIDlet
{
  private final MazeCanvas canvas3D;
  private final Image logo;
  private MenuList menuList;

  //Construct the midlet
  public Maze3DMIDlet()
  {
    logo = makeImage("/logo.png");
    canvas3D = new MazeCanvas(this);
    ErrorScreen.init(logo, Display.getDisplay(this));
  }


  public void startApp()
  {
    Displayable current = Display.getDisplay(this).getCurrent();
    if (current == null)
    {
      // check that the API is available
      boolean isApiAvailable =
        (System.getProperty("microedition.m3g.version") != null);
      menuList = new MenuList(this, isApiAvailable);
      if (!isApiAvailable)
      {
        quitApp();
      }
      else
      {
        // Display a splash screen and then the game
        StringBuffer splashText = new StringBuffer("")
          .append(getAppProperty("MIDlet-Name")).
          append("\n").append(getAppProperty("MIDlet-Vendor"));
        Alert splashScreen = new Alert("Maze3D",
          splashText.toString(),
          logo,
          AlertType.INFO);
        splashScreen.setTimeout(3000);

        Display.getDisplay(this).setCurrent(splashScreen, menuList);
      }
    }
    else
    {
      // In case the MIDlet has been hidden
      if (current == canvas3D)
      {
        canvas3D.start();
      }
      Display.getDisplay(this).setCurrent(current);
    }
  }


  public void pauseApp()
  {
    canvas3D.stop();
  }


  public void destroyApp(boolean unconditional)
  {
    canvas3D.stop();
  }

  // gets informed when the canvas itself switches the view
  void viewSwitched()
  {
    menuList.viewSwitched();
  }

  // starts a new ga,e
  void newGame()
  {
      try {
        canvas3D.stop();
        canvas3D.init();
        Display.getDisplay(this).setCurrent(canvas3D);      
      }
      catch (Exception e) {
        showError("Exception", e.getMessage());
      }
  }

  // shows the menu thread
  void showMenu()
  {
    canvas3D.stop();
    Display.getDisplay(this).setCurrent(menuList);
  }

  // quits the app but first stops the game thread
  void quitApp()
  {
    canvas3D.stop();
    notifyDestroyed();
  }

  // shows the Graphics3D properties list
  void show3DProperties()
  {
    Graphics3D g3d = Graphics3D.getInstance();
    Display.getDisplay(this).setCurrent(
      new Graphics3DProperties(g3d, this));
  }

  // switches the canvas' view
  void switchView()
  {
    canvas3D.switchView();
    canvas3D.start();
    Display.getDisplay(this).setCurrent(canvas3D);
  }

  // shows the main canvas
  void showMain()
  {
    canvas3D.start();
    Display.getDisplay(this).setCurrent(canvas3D);
  }


  // loads a given image by name
  static Image makeImage(String filename)
  {
    Image image = null;

    try
    {
      image = Image.createImage(filename);
    }
    catch (Exception e)
    {
      // use a null image instead
    }

    return image;
  }

    protected void showError(String title, String text) {
        Alert alert = new Alert(title, text, null, AlertType.ERROR);
        alert.setTimeout(Alert.FOREVER);
        Display.getDisplay(this).setCurrent(alert);
    }  
}

