// Copyright 2003 Nokia Corporation.
//
// THIS SOURCE CODE IS PROVIDED 'AS IS', WITH NO WARRANTIES WHATSOEVER,
// EXPRESS OR IMPLIED, INCLUDING ANY WARRANTY OF MERCHANTABILITY, FITNESS
// FOR ANY PARTICULAR PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE
// OR TRADE PRACTICE, RELATING TO THE SOURCE CODE OR ANY WARRANTY OTHERWISE
// ARISING OUT OF ANY PROPOSAL, SPECIFICATION, OR SAMPLE AND WITH NO
// OBLIGATION OF NOKIA TO PROVIDE THE LICENSEE WITH ANY MAINTENANCE OR
// SUPPORT. FURTHERMORE, NOKIA MAKES NO WARRANTY THAT EXERCISE OF THE
// RIGHTS GRANTED HEREUNDER DOES NOT INFRINGE OR MAY NOT CAUSE INFRINGEMENT
// OF ANY PATENT OR OTHER INTELLECTUAL PROPERTY RIGHTS OWNED OR CONTROLLED
// BY THIRD PARTIES
//
// Furthermore, information provided in this source code is preliminary,
// and may be changed substantially prior to final release. Nokia Corporation
// retains the right to make changes to this source code at
// any time, without notice. This source code is provided for informational
// purposes only.
//
// Nokia and Nokia Connecting People are registered trademarks of Nokia
// Corporation.
// Java and all Java-based marks are trademarks or registered trademarks of
// Sun Microsystems, Inc.
// Other product and company names mentioned herein may be trademarks or
// trade names of their respective owners.
//
// A non-exclusive, non-transferable, worldwide, limited license is hereby
// granted to the Licensee to download, print, reproduce and modify the
// source code. The licensee has the right to market, sell, distribute and
// make available the source code in original or modified form only when
// incorporated into the programs developed by the Licensee. No other
// license, express or implied, by estoppel or otherwise, to any other
// intellectual property rights is granted herein.
import javax.microedition.m3g.*;

class Plane
{
  // define a square in plane xy
  private final static short POINTS[] = new short[]
  {
    (short) 1, (short) 1, (short) 0,
    (short) 1, (short) -1, (short) 0,
    (short) -1, (short) 1, (short) 0,
    (short) -1, (short) -1, (short) 0,
  };

  // define the texture coordinates
  private final static short TEXTCOORDINATES[] = new short[]
  {
    (short) 1, (short) 1,
    (short) 0, (short) 1,
    (short) 1, (short) 0,
    (short) 0, (short) 0,
  };

  // triangle strip indices
  private final static int INDICES[] =
  {
    2, 3, 0,
    1, 0, 3,
    0, 3, 2,
    3, 0, 1
  };

  // strip lengths
  private final static int[] LENGTHS = new int[] {3, 3, 3, 3};

  // these arrays are the same for each plane
  private final static VertexArray POSITIONS_ARRAY, TEXTURE_ARRAY;
  private final static IndexBuffer INDEX_BUFFER;

  private Transform wallTransform = new Transform();
  private float textureRepeat;


  static
  {
    // initialize the common arrays
    POSITIONS_ARRAY = new VertexArray(POINTS.length / 3, 3, 2);
    POSITIONS_ARRAY.set(0, POINTS.length / 3, POINTS);
    TEXTURE_ARRAY = new VertexArray(TEXTCOORDINATES.length / 2, 2, 2);
    TEXTURE_ARRAY.set(0, TEXTCOORDINATES.length / 2, TEXTCOORDINATES);
    INDEX_BUFFER = new TriangleStripArray(INDICES, LENGTHS);
  }

  // Builds a new plane with a given transform)
  // and the texture repeated n times
  Plane(Transform wallTransform, float textureRepeat)
  {
    this.wallTransform = wallTransform;
    this.textureRepeat = textureRepeat;
  }


  // Build the mesh
  Mesh createMesh()
  {
    VertexBuffer vertexBuffer = new VertexBuffer();
    vertexBuffer.setPositions(POSITIONS_ARRAY, 1.0f, null);
    vertexBuffer.setTexCoords(0,
                              TEXTURE_ARRAY,
                              (float) textureRepeat, null);

    Mesh mesh = new Mesh(vertexBuffer, INDEX_BUFFER, null);
    mesh.setTransform(wallTransform);
    return mesh;
  }

}

